@@ -15,10 +15,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-''' This module represents a backup. '''
+"""
+This module represents a backup.
+"""
 
 from glob import glob
 import datetime
+from io import StringIO
 import logging
 import os
 import shutil
@@ -27,13 +30,16 @@ import tempfile
 import re
 
 import dateutil.parser
+import dateutil.tz
 
 from barman.infofile import WalFileInfo, BackupInfo, UnknownBackupIdException
 from barman.fs import UnixLocalCommand, UnixRemoteCommand, FsOperationFailed
 from barman import xlog, output
-from barman.command_wrappers import RsyncPgData, CommandFailedException
+from barman.command_wrappers import Rsync, RsyncPgData, CommandFailedException
 from barman.compression import CompressionManager, CompressionIncompatibility
 from barman.hooks import HookScriptRunner
+from barman.utils import human_readable_timedelta, mkpath
+from barman.config import BackupOptions
 
 
 _logger = logging.getLogger(__name__)
@@ -54,7 +60,9 @@ class BackupManager(object):
             'ssl_key_file', 'ssl_ca_file', 'ssl_crl_file',
             'unix_socket_directory']
     def __init__(self, server):
-        '''Constructor'''
+        """
+        Constructor
+        """
         self.name = "default"
         self.server = server
         self.config = server.config
@@ -176,34 +184,70 @@ class BackupManager(object):
         next_backup = self.get_next_backup(backup.backup_id)
         # remove the backup
         self.delete_basebackup(backup)
+        # We are deleting the first available backup
         if not previous_backup:
-            # If this is the first avalaible backup, remove unused WAL files
-            # If no backup left, remove all WAL files unless in concurrent_backup mode
-            # If in concurrent_backup mode, remove all WAL files prior to the
-            # backup that is being deleted
+            # In the case of exclusive backup (default), removes any WAL
+            # files associated to the backup being deleted.
+            # In the case of concurrent backup, removes only WAL files
+            # prior to the start of the backup being deleted, as they
+            # might be useful to any concurrent backup started immediately
+            # after.
             remove_until = None # means to remove all WAL files
             if next_backup:
                 remove_until = next_backup
-            elif self.config.backup_options == 'concurrent_backup':
+            elif BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
                 remove_until = backup
             yield "Delete associated WAL segments:"
             for name in self._remove_unused_wal_files(remove_until):
                 yield "\t%s" % name
         yield "Done"
 
-    def backup(self, immediate_checkpoint):
+    def retry_backup_copy(self, target_function, *args, **kwargs):
+        """
+        Execute the copy of a base backup, retrying a given number of times
+
+        :param target_function: the base backup copy function
+        :param args: args for the copy function
+        :param kwargs: kwargs of the copy function
+        :return: the result of the copy function
+        """
+        attempts = 0
+        while True:
+            try:
+                # if is not the first attempt, output the retry number
+                if attempts >= 1:
+                    output.warning("Copy of base backup: retry #%s", attempts)
+                return target_function(*args, **kwargs)
+            # catch rsync errors
+            except DataTransferFailure, e:
+                # exit condition: if retry number is lower than configured retry
+                # limit, try again; otherwise exit.
+                if attempts < self.config.basebackup_retry_times:
+                    # Log the exception, for debugging purpose
+                    _logger.exception("Failure in base backup copy: %s", e)
+                    output.warning(
+                        "Copy of base backup failed, waiting for next "
+                        "attempt in %s seconds",
+                        self.config.basebackup_retry_sleep)
+                    # sleep for configured time. then try again
+                    time.sleep(self.config.basebackup_retry_sleep)
+                    attempts += 1
+                else:
+                    # if the max number of attempts is reached an there is still
+                    # an error, exit re-raising the exception.
+                    raise
+
+    def backup(self):
         """
         Performs a backup for the server
-        :param immediate_checkpoint: immediate checkpoint on start_backup
         """
         _logger.debug("initialising backup information")
-        backup_stamp = datetime.datetime.now()
         self.current_action = "starting backup"
         backup_info = None
         try:
             backup_info = BackupInfo(
                 self.server,
-                backup_id=backup_stamp.strftime('%Y%m%dT%H%M%S'))
+                backup_id=datetime.datetime.now().strftime('%Y%m%dT%H%M%S'))
             backup_info.save()
             output.info(
                 "Starting backup for server %s in %s",
@@ -215,26 +259,30 @@ class BackupManager(object):
             script.env_from_backup_info(backup_info)
             script.run()
 
-            # Start the backup
-            self.backup_start(backup_info, immediate_checkpoint)
-            backup_info.set_attribute("begin_time", backup_stamp)
-            backup_info.save()
+            # Start the backup, all the subsequent code must be wrapped in a
+            # try except block which finally issue a backup_stop command
+            self.backup_start(backup_info)
+            try:
+                # save any metadata changed by backup_start() call
+                # This must be inside the try-except, because it could fail
+                backup_info.save()
 
-            # If we are the first backup, purge unused WAL files
-            previous_backup = self.get_previous_backup(backup_info.backup_id)
-            if not previous_backup:
-                self._remove_unused_wal_files(backup_info)
+                # If we are the first backup, purge unused WAL files
+                previous_backup = self.get_previous_backup(backup_info.backup_id)
+                if not previous_backup:
+                    self._remove_unused_wal_files(backup_info)
 
-            output.info("Backup start at xlog location: %s (%s, %08X)",
+                output.info("Backup start at xlog location: %s (%s, %08X)",
                             backup_info.begin_xlog,
                             backup_info.begin_wal,
                             backup_info.begin_offset)
 
-            self.current_action = "copying files"
-            try:
                 # Start the copy
+                self.current_action = "copying files"
                 output.info("Copying files.")
-                backup_size = self.backup_copy(backup_info)
+                # perform the backup copy, honouring the retry option if set
+                backup_size = self.retry_backup_copy(self.backup_copy,
+                                                     backup_info)
                 backup_info.set_attribute("size", backup_size)
                 output.info("Copy done.")
             except:
@@ -247,23 +295,27 @@ class BackupManager(object):
             finally:
                 self.backup_stop(backup_info)
 
-            if self.config.backup_options == 'concurrent_backup':
+            if BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
                 self.current_action = "writing backup label"
                 self._write_backup_label(backup_info)
             backup_info.set_attribute("status", "DONE")
-
-        except Exception, e:
+        # Use BaseException instead of Exception to catch events like
+        # KeyboardInterrupt (e.g.: CRTL-C)
+        except BaseException, e:
             msg_lines = str(e).strip().splitlines()
             if backup_info:
                 # Use only the first line of exception message
                 # in backup_info error field
                 backup_info.set_attribute("status", "FAILED")
+                # If the exception has no attached message use the raw type name
+                if len(msg_lines) == 0:
+                    msg_lines = [type(e).__name__]
                 backup_info.set_attribute(
                     "error",
                     "failure %s (%s)" % (
                         self.current_action, msg_lines[0]))
 
-            output.error("Backup failed %s: %s\n%s",
+            output.exception("Backup failed %s: %s\n%s",
                          self.current_action, msg_lines[0],
                          '\n'.join(msg_lines[1:]))
 
@@ -398,11 +450,19 @@ class BackupManager(object):
         if target_time:
             try:
                 target_datetime = dateutil.parser.parse(target_time)
-            except ValueError, e:
+            except ValueError as e:
                 msg = "unable to parse the target time parameter %r: %s" % (
                       target_time, e)
                 _logger.exception(msg)
                 raise SystemExit(msg)
+            except Exception:
+                # this should not happen, but there is a known bug in
+                # dateutil.parser.parse() implementation
+                # ref: https://bugs.launchpad.net/dateutil/+bug/1247643
+                msg = "unable to parse the target time parameter %r" % (
+                      target_time)
+                _logger.exception(msg)
+                raise SystemExit(msg)
             target_epoch = time.mktime(target_datetime.timetuple()) + (
                 target_datetime.microsecond / 1000000.)
         if target_time or target_xid or (
@@ -420,13 +480,50 @@ class BackupManager(object):
                 (", ".join(["%s: %r" % (k, v) for k, v in targets.items()]))
             wal_dest = os.path.join(dest, 'barman_xlog')
 
-        # Copy the base backup
-        msg = "Copying the base backup."
-        yield msg
-        _logger.info(msg)
+        # Retrieve the safe_horizon for smart copy
+        # If the target directory contains a previous recovery, it is safe to
+        # pick the least of the two backup "begin times" (the one we are
+        # recovering now and the one previously recovered in the target
+        # directory)
+        #
+        # noinspection PyBroadException
         try:
-            self.recover_basebackup_copy(backup, dest, tablespaces,
-                                         remote_command)
+            backup_begin_time = backup.begin_time
+            # Retrieve previously recovered backup metadata (if available)
+            dest_info_txt = cmd.get_file_content(
+                os.path.join(dest, '.barman-recover.info'))
+            dest_info = BackupInfo(
+                self.server,
+                info_file=StringIO(dest_info_txt))
+            dest_begin_time = dest_info.begin_time
+            # Pick the earlier begin time. Both are tz-aware timestamps because
+            # BackupInfo class ensure it
+            safe_horizon = min(backup_begin_time, dest_begin_time)
+            output.info("Using safe horizon time for smart rsync copy: %s",
+                        safe_horizon)
+        except FsOperationFailed, e:
+            # Setting safe_horizon to None will effectively disable
+            # the time-based part of smart_copy method. However it is still
+            # faster than running all the transfers with checksum enabled.
+            #
+            # FsOperationFailed means the .barman-recover.info is not available
+            # on destination directory
+            safe_horizon = None
+            _logger.warning('Unable to retrieve safe horizon time '
+                            'for smart rsync copy: %s', e)
+        except Exception, e:
+            # Same as above, but something failed decoding .barman-recover.info
+            # or comparing times, so log the full traceback
+            safe_horizon = None
+            _logger.exception('Error retrieving safe horizon time '
+                              'for smart rsync copy: %s', e)
+
+        # Copy the base backup
+        output.info("Copying the base backup.")
+        try:
+            # perform the backup copy, honoring the retry option if set
+            self.retry_backup_copy(self.recover_basebackup_copy, backup, dest,
+                                   tablespaces, remote_command, safe_horizon)
         except DataTransferFailure, e:
             raise SystemExit("Failure copying base backup: %s" % (e,))
         _logger.info("Base backup copied.")
@@ -455,7 +552,6 @@ class BackupManager(object):
             self.recover_xlog_copy(compressor, xlogs, wal_dest, remote_command)
         except DataTransferFailure, e:
             raise SystemExit("Failure copying WAL files: %s" % (e,))
-        _logger.info("Wal segments copied.")
 
         # Generate recovery.conf file (only if needed by PITR)
         if target_time or target_xid or (
@@ -469,7 +565,8 @@ class BackupManager(object):
             else:
                 recovery = open(os.path.join(dest, 'recovery.conf'), 'w')
             print >> recovery, "restore_command = 'cp barman_xlog/%f %p'"
-            print >> recovery, "recovery_end_command = 'rm -fr barman_xlog'"
+            if backup.version >= 80400:
+                print >> recovery, "recovery_end_command = 'rm -fr barman_xlog'"
             if target_time:
                 print >> recovery, "recovery_target_time = '%s'" % target_time
             if target_tli:
@@ -483,9 +580,14 @@ class BackupManager(object):
                     not exclusive)
             recovery.close()
             if remote_command:
+                # Uses plain rsync (without exclusions) to ship recovery.conf
+                plain_rsync = Rsync(
+                        ssh=remote_command,
+                        bwlimit=self.config.bandwidth_limit,
+                        network_compression=self.config.network_compression)
                 try:
-                    rsync.from_file_list(['recovery.conf'],
-                                         tempdir, ':%s' % dest)
+                    plain_rsync.from_file_list(['recovery.conf'],
+                                              tempdir, ':%s' % dest)
                 except CommandFailedException, e:
                     msg = (
                         'remote copy of recovery.conf failed: %s' % (e,))
@@ -500,7 +602,7 @@ class BackupManager(object):
                 status_dir = tempfile.mkdtemp(prefix='barman_xlog_status-')
             else:
                 status_dir = os.path.join(wal_dest, 'archive_status')
-                os.makedirs(status_dir) # no need to check, it must not exist
+                mkpath(status_dir)
             for filename in required_xlog_files:
                 with open(os.path.join(status_dir, "%s.done" % filename),
                           'a') as f:
@@ -546,6 +648,17 @@ class BackupManager(object):
                 raise SystemExit(msg)
             shutil.rmtree(tempdir)
 
+        # Copy the backup.info file to the destination as ".barman-recover.info"
+        if remote_command:
+            try:
+                rsync(backup.filename, ':%s/.barman-recover.info' % dest)
+            except CommandFailedException, e:
+                    msg = 'copy of recovery metadata file failed: %s' % (e,)
+                    _logger.error(msg)
+                    raise SystemExit(msg)
+        else:
+            backup.save(os.path.join(dest, '.barman-recover.info'))
+
         yield ""
         yield "Your PostgreSQL server has been successfully prepared for " \
               "recovery!"
@@ -554,10 +667,21 @@ class BackupManager(object):
               "PostgreSQL"
         yield "configuration file before starting the just recovered instance."
         yield ""
+        # With a PostgreSQL version older than 8.4, it is the user's
+        # responsibility to delete the "barman_xlog" directory as the
+        # restore_command option in recovery.conf is not supported
+        if backup.version < 80400 and (target_time or target_xid or (
+                target_tli and target_tli != backup.timeline) or target_name):
+            yield "After the recovery, please remember to remove the " \
+                  "\"barman_xlog\" directory"
+            yield "inside the PostgreSQL data directory."
+            yield ""
         if clashes:
             yield "WARNING: Before starting up the recovered PostgreSQL server,"
-            yield "please review also the settings of the following configuration"
-            yield "options as they might interfere with your current recovery attempt:"
+            yield "please review also the settings of the following " \
+                  "configuration"
+            yield "options as they might interfere with your current " \
+                  "recovery attempt:"
             yield ""
 
             for name, value in sorted(clashes.items()):
@@ -579,32 +703,40 @@ class BackupManager(object):
         compressor = self.compression_manager.get_compressor()
         with self.server.xlogdb('a') as fxlogdb:
             if verbose:
-                output.info("Processing xlog segments for %s", self.config.name)
+                output.info("Processing xlog segments for %s",
+                            self.config.name,
+                            log=False)
             available_backups = self.get_available_backups(
                 BackupInfo.STATUS_ALL)
             for filename in sorted(glob(
                     os.path.join(self.config.incoming_wals_directory, '*'))):
                 if not found and not verbose:
                     output.info("Processing xlog segments for %s",
-                                self.config.name)
+                                self.config.name,
+                                log=False)
                 found = True
                 # Delete xlog segments only if the backup is exclusive
                 if (not len(available_backups) and
-                        (self.config.backup_options != "concurrent_backup")):
+                        (BackupOptions.CONCURRENT_BACKUP not in
+                             self.config.backup_options)):
                     output.info("\tNo base backup available. Trashing file %s"
                                 " from server %s",
                                 os.path.basename(filename), self.config.name)
                     os.unlink(filename)
                     continue
+                # Report to the user the WAL file we are archiving
+                output.info("\t%s", os.path.basename(filename), log=False)
+                _logger.info("Archiving %s/%s",
+                             self.config.name,
+                             os.path.basename(filename))
                 # Archive the WAL file
                 wal_info = self.cron_wal_archival(compressor, filename)
 
                 # Updates the information of the WAL archive with
                 # the latest segments
                 fxlogdb.write(wal_info.to_xlogdb_line())
-                output.info("\t%s", os.path.basename(filename))
         if not found and verbose:
-            output.info("\tno file found")
+            output.info("\tno file found", log=False)
 
         # Retention policy management
         if (self.server.enforce_retention_policies
@@ -629,17 +761,17 @@ class BackupManager(object):
         backup_dir = backup.get_basebackup_directory();
         shutil.rmtree(backup_dir)
 
-    def delete_wal(self, name):
-        '''
+    def delete_wal(self, wal_info):
+        """
         Delete a WAL segment, with the given name
 
         :param name: the name of the WAL to delete
-        '''
-        hashdir = os.path.join(self.config.wals_directory, xlog.hash_dir(name))
+        """
+
         try:
-            os.unlink(os.path.join(hashdir, name))
+            os.unlink(wal_info.full_path)
             try:
-                os.removedirs(hashdir)
+                os.removedirs(os.path.dirname(wal_info.full_path))
             except OSError:
                 # This is not an error condition
                 # We always try to remove the the trailing directories,
@@ -647,15 +779,13 @@ class BackupManager(object):
                 pass
         except OSError:
             _logger.warning('Expected WAL file %s not found during delete',
-                            name, exc_info=1)
+                            wal_info.name, exc_info=1)
 
-    def backup_start(self, backup_info, immediate_checkpoint):
+    def backup_start(self, backup_info):
         """
         Start of the backup
 
-        :param immediate_checkpoint: force execution of a check point as soon
-        as possible
-        :param backup_info: the backup information structure
+        :param BackupInfo backup_info: the backup information
         """
         self.current_action = "connecting to database (%s)" % self.config.conninfo
         _logger.debug(self.current_action)
@@ -690,7 +820,7 @@ class BackupManager(object):
         _logger.debug(self.current_action)
         label = "Barman backup %s %s" % (
             backup_info.server_name, backup_info.backup_id)
-        self.server.start_backup(label, immediate_checkpoint, backup_info)
+        self.server.start_backup(label, backup_info)
 
     def _raise_rsync_error(self, e, msg):
         """
@@ -751,7 +881,7 @@ class BackupManager(object):
         if len(tablespaces_bwlimit) > 0:
             # we are copying the tablespaces before the data directory,
             # so we need to create the 'pg_tblspc' directory
-            os.makedirs(os.path.join(backup_dest, 'pg_tblspc'))
+            mkpath(os.path.join(backup_dest, 'pg_tblspc'))
             for tablespace_dir, bwlimit in tablespaces_bwlimit.items():
                 self.current_action = "copying tablespace '%s' with bwlimit " \
                                       "%d" % (tablespace_dir, bwlimit)
@@ -820,13 +950,22 @@ class BackupManager(object):
                                   cf[key]
                             self._raise_rsync_error(e, msg)
 
+        # Calculate the base backup size
         self.current_action = "calculating backup size"
         _logger.debug(self.current_action)
         backup_size = 0
         for dirpath, _, filenames in os.walk(backup_dest):
-            for f in filenames:
-                fp = os.path.join(dirpath, f)
-                backup_size += os.path.getsize(fp)
+            # execute fsync() on the containing directory
+            dir_fd = os.open(dirpath, os.O_DIRECTORY)
+            os.fsync(dir_fd)
+            os.close(dir_fd)
+            # execute fsync() on all the contained files
+            for filename in filenames:
+                file_path = os.path.join(dirpath, filename)
+                file_fd = os.open(file_path, os.O_RDONLY)
+                backup_size += os.fstat(file_fd).st_size
+                os.fsync(file_fd)
+                os.close(file_fd)
         return backup_size
 
     def backup_stop(self, backup_info):
@@ -837,7 +976,8 @@ class BackupManager(object):
         '''
         self.server.stop_backup(backup_info)
 
-    def recover_basebackup_copy(self, backup, dest, tablespaces, remote_command=None):
+    def recover_basebackup_copy(self, backup, dest, tablespaces,
+                                remote_command=None, safe_horizon=None):
         """
         Perform the actual copy of the base backup for recovery purposes
 
@@ -846,6 +986,8 @@ class BackupManager(object):
         :param tablespaces: tablespace relocation options
         :param remote_command: default None. The remote command to recover
                                the base backup, in case of remote backup.
+        :param datetime.datetime safe_horizon: anything after this time
+            has to be checked with checksums
         """
 
         sourcedir = os.path.join(backup.get_basebackup_directory(), 'pgdata')
@@ -894,7 +1036,7 @@ class BackupManager(object):
             exclude_and_protect=exclude_and_protect,
             network_compression=self.config.network_compression)
         try:
-            rsync('%s/' % (sourcedir,), dest)
+            rsync.smart_copy('%s/' % (sourcedir,), dest, safe_horizon)
         except CommandFailedException, e:
             msg = "data transfer failure on directory '%s'" % (dest[1:],)
             self._raise_rsync_error(e, msg)
@@ -907,8 +1049,10 @@ class BackupManager(object):
                     bwlimit=bwlimit,
                     network_compression=self.config.network_compression)
                 try:
-                    tb_rsync('%s/' % os.path.join(sourcedir, tablespace_dir),
-                             os.path.join(dest, tablespace_dir))
+                    tb_rsync.smart_copy(
+                        '%s/' % os.path.join(sourcedir, tablespace_dir),
+                        os.path.join(dest, tablespace_dir),
+                        safe_horizon)
                 except CommandFailedException, e:
                     msg = "data transfer failure on directory '%s'" % (
                         tablespace_dir[1:],)
@@ -934,15 +1078,27 @@ class BackupManager(object):
             network_compression=self.config.network_compression)
         if remote_command:
             # If remote recovery tell rsync to copy them remotely
-            wal_dest = ':%s' % wal_dest
+            # add ':' prefix to mark it as remote
+            # add '/' suffix to ensure it is a directory
+            wal_dest = ':%s/' % wal_dest
         else:
             # we will not use rsync: destdir must exists
-            if not os.path.exists(wal_dest):
-                os.makedirs(wal_dest)
+            mkpath(wal_dest)
         if compressor and remote_command:
             xlog_spool = tempfile.mkdtemp(prefix='barman_xlog-')
-        for prefix in xlogs:
+        total_wals = sum(map(len, xlogs.values()))
+        partial_count = 0
+        for prefix in sorted(xlogs):
+            batch_len = len(xlogs[prefix])
+            partial_count += batch_len
             source_dir = os.path.join(self.config.wals_directory, prefix)
+            _logger.info(
+                "Starting copy of %s WAL files %s/%s from %s to %s",
+                batch_len,
+                partial_count,
+                total_wals,
+                xlogs[prefix][0],
+                xlogs[prefix][-1])
             if compressor:
                 if remote_command:
                     for segment in xlogs[prefix]:
@@ -976,6 +1132,8 @@ class BackupManager(object):
                           "to directory '%s'" % (wal_dest[1:],)
                     self._raise_rsync_error(e, msg)
 
+        _logger.info("Finished copying %s WAL files.", total_wals)
+
         if compressor and remote_command:
             shutil.rmtree(xlog_spool)
 
@@ -999,8 +1157,7 @@ class BackupManager(object):
         script.env_from_wal_info(wal_info)
         script.run()
 
-        if not os.path.isdir(destdir):
-            os.makedirs(destdir)
+        mkpath(destdir)
         if compressor:
             compressor.compress(filename, destfile)
             shutil.copystat(filename, destfile)
@@ -1008,6 +1165,15 @@ class BackupManager(object):
         else:
             shutil.move(filename, destfile)
 
+        # execute fsync() on the archived WAL containing directory
+        dir_fd = os.open(os.path.dirname(destfile), os.O_DIRECTORY)
+        os.fsync(dir_fd)
+        os.close(dir_fd)
+        # execute fsync() on the archived WAL file
+        file_fd = os.open(destfile, os.O_RDONLY)
+        os.fsync(file_fd)
+        os.close(file_fd)
+
         wal_info = WalFileInfo.from_file(
             destfile,
             compression=compressor and compressor.compression)
@@ -1226,13 +1392,46 @@ class BackupManager(object):
             xlogdb_new = fxlogdb.name + ".new"
             with open(xlogdb_new, 'w') as fxlogdb_new:
                 for line in fxlogdb:
-                    name, _, _, _ = self.server.xlogdb_parse_line(line)
-                    if backup_info and name >= backup_info.begin_wal:
-                        fxlogdb_new.write(line)
+                    wal_info = WalFileInfo.from_xlogdb_line(self.server, line)
+                    if backup_info and wal_info.name >= backup_info.begin_wal:
+                        fxlogdb_new.write(wal_info.to_xlogdb_line())
                         continue
                     else:
                         # Delete the WAL segment
-                        self.delete_wal(name)
-                        removed.append(name)
+                        self.delete_wal(wal_info)
+                        removed.append(wal_info.name)
             shutil.move(xlogdb_new, fxlogdb.name)
         return removed
+
+    def validate_last_backup_maximum_age(self, last_backup_maximum_age):
+        """
+        Evaluate the age of the last available backup in a catalogue.
+        If the last backup is older than the specified time interval (age),
+        the function returns False. If within the requested age interval,
+        the function returns True.
+
+        :param timedate.timedelta last_backup_maximum_age: time interval
+            representing the maximum allowed age for the last backup in a server
+            catalogue
+        :return tuple: a tuple containing the boolean result of the check and
+            auxiliary information about the last backup current age
+        """
+        # Get the ID of the last available backup
+        backup_id = self.get_last_backup()
+        if backup_id:
+            # Get the backup object
+            backup = BackupInfo(self.server, backup_id=backup_id)
+            now = datetime.datetime.now(dateutil.tz.tzlocal())
+            # Evaluate the point of validity
+            validity_time = now - last_backup_maximum_age
+            # Pretty print of a time interval (age)
+            msg = human_readable_timedelta(now - backup.end_time)
+            # If the backup end time is older than the point of validity,
+            # return False, otherwise return true
+            if backup.end_time < validity_time:
+                return False, msg
+            else:
+                return True, msg
+        else:
+            # If no backup is available return false
+            return False, "No available backups"
