@@ -18,6 +18,7 @@ import ast
 
 import os
 import dateutil.parser
+import dateutil.tz
 import collections
 from barman import xlog
 from barman.compression import identify_compression
@@ -31,7 +32,7 @@ def output_tablespace_list(tablespaces):
     """
     Return the literal representation of tablespaces as a Python string
 
-    :param tablespaces list: list of Tablespaces objects
+    :param tablespaces tablespaces: list of Tablespaces objects
     :return str: Literal representation of tablespaces
     """
     if tablespaces:
@@ -56,6 +57,25 @@ def load_tablespace_list(string):
         return None
 
 
+def load_datetime_tz(time_str):
+    """
+    Load datetime and ensure the result is timezone-aware.
+
+    If the parsed timestamp is naive, transform it into a timezone-aware one
+    using the local timezone.
+
+    :param str time_str: string representing a timestamp
+    :return datetime: the parsed timezone-aware datetime
+    """
+    # dateutil parser returns naive or tz-aware string depending on the format
+    # of the input string
+    timestamp = dateutil.parser.parse(time_str)
+    # if the parsed timestamp is naive, forces it to local timezone
+    if timestamp.tzinfo is None:
+        timestamp = timestamp.replace(tzinfo=dateutil.tz.tzlocal())
+    return timestamp
+
+
 class Field(object):
 
     def __init__(self, name, dump=None, load=None, default=None, doc=None):
@@ -86,6 +106,7 @@ class Field(object):
         self.default = default
         self.__doc__ = doc
 
+    # noinspection PyUnusedLocal
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
@@ -269,7 +290,8 @@ class WalFileInfo(FieldListFile):
     name = Field('name', doc='base name of WAL file')
     full_path = Field('full_path', doc='complete path of the file')
     size = Field('size', load=int, doc='WAL file size after compression')
-    time = Field('time', load=int, doc='WAL file modification time')
+    time = Field('time', load=float, doc='WAL file modification time '
+                                         '(seconds since epoch)')
     compression = Field('compression', doc='compression type')
 
     @classmethod
@@ -307,6 +329,33 @@ class WalFileInfo(FieldListFile):
             self.time,
             self.compression)
 
+    @classmethod
+    def from_xlogdb_line(cls, server, line):
+        """
+        Parse a line from xlog catalogue
+
+        :param Server server: server object
+        :param str line: a line in the wal database to parse
+        :rtype: WalFileInfo
+        """
+        try:
+            name, size, time, compression = line.split()
+        except ValueError:
+            # Old format compatibility (no compression)
+            compression = None
+            try:
+                name, size, time = line.split()
+            except ValueError:
+                raise ValueError("cannot parse line: %r" % (line,))
+        # The to_xlogdb_line method writes None values as literal 'None'
+        if compression == 'None':
+            compression = None
+        full_path = server.get_wal_full_path(name)
+        size = int(size)
+        time = float(time)
+        return cls(name=name, full_path=full_path, size=size, time=time,
+                   compression=compression)
+
 
 class UnknownBackupIdException(Exception):
     """
@@ -345,12 +394,12 @@ class BackupInfo(FieldListFile):
                         dump=output_tablespace_list)
     # Timeline is an integer
     timeline = Field('timeline', load=int)
-    begin_time = Field('begin_time', load=dateutil.parser.parse)
+    begin_time = Field('begin_time', load=load_datetime_tz)
     begin_xlog = Field('begin_xlog')
     begin_wal = Field('begin_wal')
     begin_offset = Field('begin_offset')
     size = Field('size', load=int)
-    end_time = Field('end_time', load=dateutil.parser.parse)
+    end_time = Field('end_time', load=load_datetime_tz)
     end_xlog = Field('end_xlog')
     end_wal = Field('end_wal')
     end_offset = Field('end_offset')
@@ -427,20 +476,19 @@ class BackupInfo(FieldListFile):
         if target in 'standalone':
             # List all the WAL files for this backup
             for x in self.get_required_wal_segments():
-                hash_dir = os.path.join(self.config.wals_directory,
-                                        xlog.hash_dir(x))
-                yield os.path.join(hash_dir, x)
+                yield self.server.get_wal_full_path(x)
         if target in ('wal', 'full'):
-            for x, _ in self.server.get_wal_until_next_backup(self):
-                hash_dir = os.path.join(self.config.wals_directory,
-                                        xlog.hash_dir(x))
-                yield os.path.join(hash_dir, x)
+            for wal_info in self.server.get_wal_until_next_backup(self):
+                yield wal_info.full_path
 
     def detect_backup_id(self):
         """
         Detect the backup ID from the name of the parent dir of the info file
         """
-        return os.path.basename(os.path.dirname(self.filename))
+        if self.filename:
+            return os.path.basename(os.path.dirname(self.filename))
+        else:
+            return None
 
     def get_basebackup_directory(self):
         """
