@@ -423,7 +423,6 @@ class ConsoleOutputWriter(object):
         """
         Output a single backup in the list-backup command
 
-        :param basestring server_name: server we are displaying
         :param BackupInfo backup_info: backup we are displaying
         :param backup_size: size of base backup (with the required WAL files)
         :param wal_size: size of WAL files belonging to this backup
@@ -435,8 +434,8 @@ class ConsoleOutputWriter(object):
             self.info(backup_info.backup_id)
             return
 
-        out_list = ["%s %s - "
-            % (backup_info.server_name, backup_info.backup_id)]
+        out_list = [
+            "%s %s - " % (backup_info.server_name, backup_info.backup_id)]
         if backup_info.status == BackupInfo.DONE:
             end_time = backup_info.end_time.ctime()
             out_list.append('%s - Size: %s - WAL Size: %s' %
@@ -464,11 +463,11 @@ class ConsoleOutputWriter(object):
         """
         data = dict(backup_ext_info)
         self.info("Backup %s:", data['backup_id'])
-        self.info("  Server Name       : %s", data['server_name'])
-        self.info("  Status            : %s", data['status'])
+        self.info("  Server Name            : %s", data['server_name'])
+        self.info("  Status                 : %s", data['status'])
         if data['status'] == BackupInfo.DONE:
-            self.info("  PostgreSQL Version: %s", data['version'])
-            self.info("  PGDATA directory  : %s", data['pgdata'])
+            self.info("  PostgreSQL Version     : %s", data['version'])
+            self.info("  PGDATA directory       : %s", data['pgdata'])
             if data['tablespaces']:
                 self.info("  Tablespaces:")
                 for item in data['tablespaces']:
@@ -476,40 +475,54 @@ class ConsoleOutputWriter(object):
                               item.name, item.location, item.oid)
             self.info("")
             self.info("  Base backup information:")
-            self.info("    Disk usage      : %s",
+            self.info("    Disk usage           : %s",
                       pretty_size(data['size'] + data[
                           'wal_size']))
-            self.info("    Timeline        : %s", data['timeline'])
-            self.info("    Begin WAL       : %s",
+            self.info("    Timeline             : %s", data['timeline'])
+            self.info("    Begin WAL            : %s",
                       data['begin_wal'])
-            self.info("    End WAL         : %s", data['end_wal'])
-            self.info("    WAL number      : %s", data['wal_num'])
-            self.info("    Begin time      : %s",
+            self.info("    End WAL              : %s", data['end_wal'])
+            self.info("    WAL number           : %s", data['wal_num'])
+            # Output WAL compression ratio for basebackup WAL files
+            if data['wal_compression_ratio'] > 0:
+                self.info("    WAL compression ratio: %s",
+                          '{percent:.2%}'.format(
+                              percent=data['wal_compression_ratio']))
+            self.info("    Begin time           : %s",
                       data['begin_time'])
-            self.info("    End time        : %s", data['end_time'])
-            self.info("    Begin Offset    : %s",
+            self.info("    End time             : %s", data['end_time'])
+            self.info("    Begin Offset         : %s",
                       data['begin_offset'])
-            self.info("    End Offset      : %s",
+            self.info("    End Offset           : %s",
                       data['end_offset'])
-            self.info("    Begin XLOG      : %s",
+            self.info("    Begin XLOG           : %s",
                       data['begin_xlog'])
-            self.info("    End XLOG        : %s", data['end_xlog'])
+            self.info("    End XLOG             : %s", data['end_xlog'])
             self.info("")
             self.info("  WAL information:")
-            self.info("    No of files     : %s",
+            self.info("    No of files          : %s",
                       data['wal_until_next_num'])
-            self.info("    Disk usage      : %s",
+            self.info("    Disk usage           : %s",
                       pretty_size(data['wal_until_next_size']))
-            self.info("    Last available  : %s", data['wal_last'])
+            # Output WAL rate
+            if data['wals_per_second'] > 0:
+                self.info("    WAL rate             : %0.2f/hour",
+                          data['wals_per_second']*3600)
+            # Output WAL compression ratio for archived WAL files
+            if data['wal_until_next_compression_ratio'] > 0:
+                self.info("    Compression ratio    : %s",
+                          '{percent:.2%}'.format(
+                              percent=data['wal_until_next_compression_ratio']))
+            self.info("    Last available       : %s", data['wal_last'])
             self.info("")
             self.info("  Catalog information:")
-            self.info("    Retention Policy: %s",
+            self.info("    Retention Policy     : %s",
                       data['retention_policy_status']
                       or 'not enforced')
-            self.info("    Previous Backup : %s",
+            self.info("    Previous Backup      : %s",
                       data.setdefault('previous_backup_id', 'not available')
                       or '- (this is the oldest base backup)')
-            self.info("    Next Backup     : %s",
+            self.info("    Next Backup          : %s",
                       data.setdefault('next_backup_id', 'not available')
                       or '- (this is the latest base backup)')
         else:
@@ -605,20 +618,75 @@ class NagiosOutputWriter(ConsoleOutputWriter):
 
         Also set the exit code as 2 (CRITICAL) in case of errors
         """
-        issues = []
+        # List of all servers that have been checked
         servers = []
+        # List of servers reporting issues
+        issues = []
         for item in self.result_check_list:
+            # Keep track of all the checked servers
             if item['server_name'] not in servers:
                 servers.append(item['server_name'])
+            # Keep track of the servers with issues
             if not item['status'] and item['server_name'] not in issues:
                 issues.append(item['server_name'])
         if len(issues) > 0:
-            print "BARMAN CRITICAL - %d server out of %d has issues" % \
-                  (len(issues), len(servers))
+            fail_summary = []
+            details = []
+            for server in issues:
+                # Join all the issues for a server. Output format is in the
+                # form:
+                # "<server_name> FAILED: <failed_check1>, <failed_check2> ... "
+                # All strings will be concatenated into the $SERVICEOUTPUT$
+                # macro of the Nagios output
+                server_fail = "%s FAILED: %s" % (
+                    server,
+                    ", ".join([
+                        item['check']
+                        for item in self.result_check_list
+                        if item['server_name'] == server and not item['status']
+                    ]))
+                fail_summary.append(server_fail)
+                # Prepare an array with the detailed output for
+                # the $LONGSERVICEOUTPUT$ macro of the Nagios output
+                # line format:
+                # <servername>.<failed_check1>: FAILED
+                # <servername>.<failed_check2>: FAILED (Hint if present)
+                # <servername2.<failed_check1>: FAILED
+                # .....
+                for issue in self.result_check_list:
+                    if issue['server_name'] == server and not issue['status']:
+                        fail_detail = "%s.%s: FAILED" % (server, issue['check'])
+                        if issue['hint']:
+                            fail_detail += " (%s)" % issue['hint']
+                        details.append(fail_detail)
+            # Append the summary of failures to the first line of the output
+            # using * as delimiter
+            if len(servers) == 1:
+                print "BARMAN CRITICAL - server %s has issues * %s" % \
+                    (servers[0], " * ".join(fail_summary))
+            else:
+                print "BARMAN CRITICAL - %d server out of %d have issues * " \
+                      "%s" % (len(issues), len(servers),
+                              " * ".join(fail_summary))
+
+            # add the detailed list to the output
+            for issue in details:
+                print issue
             global error_exit_code
             error_exit_code = 2
         else:
-            print "BARMAN OK - Ready to serve the Espresso backup"
+            # No issues, all good!
+            # Display the output message for a single server check
+            if len(servers) == 1:
+                print "BARMAN OK - Ready to serve the Espresso backup " \
+                    "for %s" % \
+                    (servers[0])
+            else:
+                # Display the output message for several servers, using
+                # '*' as delimiter
+                print "BARMAN OK - Ready to serve the Espresso backup " \
+                    "for %d server(s) * %s" % \
+                    (len(servers), " * ".join([server for server in servers]))
 
 
 #: This dictionary acts as a registry of available OutputWriters
