@@ -30,6 +30,7 @@ from datetime import datetime, timedelta
 import re
 import logging
 from barman.infofile import BackupInfo
+from dateutil import tz
 
 _logger = logging.getLogger(__name__)
 
@@ -108,6 +109,12 @@ class RetentionPolicy(object):
             'The class %s must override the create() class method',
             cls.__name__)
 
+    def to_json(self):
+        """
+        Output representation of the obj for JSON serialization
+        """
+        return "%s %s" % (self.mode, self.value)
+
 
 class RedundancyRetentionPolicy(RetentionPolicy):
     '''
@@ -221,7 +228,7 @@ class RecoveryWindowRetentionPolicy(RetentionPolicy):
         of recoverability, which will be then used to define the first
         backup or the first WAL
         '''
-        return datetime.now() - self.timedelta
+        return datetime.now(tz.tzlocal()) - self.timedelta
 
     def _backup_report(self):
         '''Report obsolete/valid backups according to the retention policy'''
