@@ -23,18 +23,23 @@ Barman is able to manage multiple servers.
 import os
 import re
 import logging
-import datetime
 from contextlib import contextmanager
 
 import psycopg2
+import sys
+import shutil
+from datetime import timedelta
 
 from barman import output
-from barman.infofile import BackupInfo, UnknownBackupIdException, Tablespace
+from barman.config import BackupOptions
+from barman.infofile import BackupInfo, UnknownBackupIdException, Tablespace, \
+    WalFileInfo
 from barman.lockfile import LockFile, LockFileBusy, \
     LockFilePermissionDenied
 from barman.backup import BackupManager
 from barman.command_wrappers import Command
 from barman.retention_policies import RetentionPolicyFactory
+from barman.utils import human_readable_timedelta
 import xlog
 
 
@@ -167,6 +172,8 @@ class Server(object):
         self.check_directories()
         # Check retention policies
         self.check_retention_policy_settings()
+        # Check for backup validity
+        self.check_backup_validity()
         # Executes the backup manager set of checks
         self.backup_manager.check()
 
@@ -193,7 +200,7 @@ class Server(object):
             output.result('check', self.config.name, 'PostgreSQL', False)
             return
 
-        if self.config.backup_options == 'concurrent_backup':
+        if BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
             if remote_status['pgespresso_installed']:
                 output.result('check', self.config.name,
                         'pgespresso extension', True)
@@ -249,6 +256,30 @@ class Server(object):
             output.result('check', self.config.name,
                           'retention policy settings', True)
 
+    def check_backup_validity(self):
+        """
+        Check if backup validity requirements are satisfied
+        """
+        # first check: check backup maximum age
+        if self.config.last_backup_maximum_age is not None:
+            # get maximum age informations
+            backup_age = self.backup_manager.validate_last_backup_maximum_age(
+                self.config.last_backup_maximum_age)
+
+            # format the output
+            output.result('check', self.config.name,
+                          'backup maximum age', backup_age[0],
+                          "interval provided: %s, latest backup age: %s" %
+                          (human_readable_timedelta(
+                              self.config.last_backup_maximum_age),
+                           backup_age[1]))
+        else:
+            # last_backup_maximum_age provided by the user
+            output.result('check', self.config.name,
+                          'backup maximum age',
+                          True,
+                          "no last_backup_maximum_age provided")
+
     def status_postgres(self):
         """
         Status of PostgreSQL server
@@ -435,6 +466,30 @@ class Server(object):
         ])
         remote_status = self.get_remote_status()
         result.update(remote_status)
+        # backup maximum age section
+        if self.config.last_backup_maximum_age is not None:
+            age = self.backup_manager.validate_last_backup_maximum_age(
+                self.config.last_backup_maximum_age)
+            # if latest backup is between the limits of the
+            # last_backup_maximum_age configuration, display how old is
+            # the latest backup.
+            if age[0]:
+                msg = "%s (latest backup: %s )" % \
+                    (human_readable_timedelta(
+                        self.config.last_backup_maximum_age),
+                     age[1])
+            else:
+                # if latest backup is outside the limits of the
+                # last_backup_maximum_age configuration (or the configuration
+                # value is none), warn the user.
+                msg = "%s (WARNING! latest backup is %s old)" % \
+                    (human_readable_timedelta(
+                        self.config.last_backup_maximum_age),
+                     age[1])
+            result['last_backup_maximum_age'] = msg
+        else:
+            result['last_backup_maximum_age'] = "None"
+
         output.result('show_server', self.config.name, result)
 
     @contextmanager
@@ -518,89 +573,94 @@ class Server(object):
                               "location: %s", e)
                 return {}
 
-    def pg_start_backup(self, backup_label, immediate_checkpoint):
+    def pg_start_backup(self, backup_label):
         """
         Execute a pg_start_backup
 
-        :param backup_label: label for the backup
-        :param immediate_checkpoint: Boolean for immediate checkpoint execution
+        :param str backup_label: label for the backup
         """
         with self.pg_connect() as conn:
-            if (self.config.backup_options != "concurrent_backup" and
-                    self.pg_is_in_recovery()):
+            if (BackupOptions.CONCURRENT_BACKUP not in
+                    self.config.backup_options and self.pg_is_in_recovery()):
                 raise Exception(
                     'Unable to start a backup because of server recovery state')
             try:
                 cur = conn.cursor()
-                cur.execute(
-                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* '
-                    'FROM pg_start_backup(%s,%s) as xlog_loc',
-                    (backup_label, immediate_checkpoint))
+                if self.server_version < 80400:
+                    cur.execute(
+                        'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).*, '
+                        'now() FROM pg_start_backup(%s) as xlog_loc',
+                        (backup_label,))
+                else:
+                    cur.execute(
+                        'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).*, '
+                        'now() FROM pg_start_backup(%s,%s) as xlog_loc',
+                        (backup_label, self.config.immediate_checkpoint))
                 return cur.fetchone()
             except psycopg2.Error, e:
                 msg = "pg_start_backup(): %s" % e
                 _logger.debug(msg)
                 raise Exception(msg)
 
-    def pgespresso_start_backup(self, backup_label, immediate_checkpoint):
+    def pgespresso_start_backup(self, backup_label):
         """
         Execute a pgespresso_start_backup
 
-        :param backup_label: label for the backup
-        :param immediate_checkpoint Boolean for immediate checkpoint execution
+        :param str backup_label: label for the backup
         """
         with self.pg_connect() as conn:
 
-            if (self.config.backup_options == "concurrent_backup" and
-                    not self.pg_espresso_installed()):
+            if (BackupOptions.CONCURRENT_BACKUP in self.config.backup_options
+                and not self.pg_espresso_installed()):
                 raise Exception(
                     'pgespresso extension required for concurrent_backup')
             try:
                 cur = conn.cursor()
-                cur.execute('SELECT pgespresso_start_backup(%s,%s)',
-                            (backup_label, immediate_checkpoint))
-                return cur.fetchone()[0]
+                cur.execute('SELECT pgespresso_start_backup(%s,%s), now()',
+                            (backup_label, self.config.immediate_checkpoint))
+                return cur.fetchone()
             except psycopg2.Error, e:
                 msg = "pgexpresso_start_backup(): %s" % e
                 _logger.debug(msg)
                 raise Exception(msg)
 
-    def start_backup(self, label, immediate_checkpoint, backup_info):
+    def start_backup(self, label, backup_info):
         """
         start backup wrapper
 
-        :param backup_label: label for the backup
-        :param immediate_checkpoint: Boolean for immediate checkpoint execution
-        :param backup_info: backup_info object
+        :param str label: label for the backup
+        :param BackupInfo backup_info: backup information object
         :return:
         """
-        if self.config.backup_options != 'concurrent_backup':
-            start_row = self.pg_start_backup(label, immediate_checkpoint)
-            if start_row:
-                start_xlog, start_file_name, start_file_offset = start_row
-                backup_info.set_attribute("status", "STARTED")
-                backup_info.set_attribute("timeline",
-                                          int(start_file_name[0:8], 16))
-                backup_info.set_attribute("begin_xlog", start_xlog)
-                backup_info.set_attribute("begin_wal", start_file_name)
-                backup_info.set_attribute("begin_offset", start_file_offset)
+        if BackupOptions.CONCURRENT_BACKUP not in self.config.backup_options:
+            start_row = self.pg_start_backup(label)
+            start_xlog, start_file_name, start_file_offset, start_time = \
+                start_row
+            backup_info.set_attribute("status", "STARTED")
+            backup_info.set_attribute("timeline",
+                                      int(start_file_name[0:8], 16))
+            backup_info.set_attribute("begin_xlog", start_xlog)
+            backup_info.set_attribute("begin_wal", start_file_name)
+            backup_info.set_attribute("begin_offset", start_file_offset)
+            backup_info.set_attribute("begin_time", start_time)
 
-        elif self.config.backup_options == 'concurrent_backup':
-            backup_data = self.pgespresso_start_backup(label,
-                                                       immediate_checkpoint)
-            if backup_data:
-                e = re.compile('^START WAL LOCATION: (.*) \(file (.*)\)',
-                               re.MULTILINE)
-                b_info = e.search(backup_data)
-                backup_info.set_attribute("status", "STARTED")
-                backup_info.set_attribute("timeline",
-                                          int(b_info.group(2)[0:8], 16))
-                backup_info.set_attribute("begin_xlog", b_info.group(1))
-                backup_info.set_attribute("begin_wal", b_info.group(2))
-                backup_info.set_attribute("begin_offset",
-                                          xlog.get_offset_from_location(
-                                              b_info.group(1)))
-                backup_info.set_attribute("backup_label", backup_data)
+        else:
+            start_row = self.pgespresso_start_backup(label)
+            backup_data, start_time = start_row
+            wal_re = re.compile(
+                '^START WAL LOCATION: (.*) \(file (.*)\)',
+                re.MULTILINE)
+            wal_info = wal_re.search(backup_data)
+            backup_info.set_attribute("status", "STARTED")
+            backup_info.set_attribute("timeline",
+                                      int(wal_info.group(2)[0:8], 16))
+            backup_info.set_attribute("begin_xlog", wal_info.group(1))
+            backup_info.set_attribute("begin_wal", wal_info.group(2))
+            backup_info.set_attribute("begin_offset",
+                                      xlog.get_offset_from_location(
+                                          wal_info.group(1)))
+            backup_info.set_attribute("backup_label", backup_data)
+            backup_info.set_attribute("begin_time", start_time)
 
     def pg_stop_backup(self):
         """
@@ -610,8 +670,8 @@ class Server(object):
             try:
                 cur = conn.cursor()
                 cur.execute(
-                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* '
-                    'from pg_stop_backup() as xlog_loc')
+                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).*, '
+                    'now() FROM pg_stop_backup() as xlog_loc')
                 return cur.fetchone()
             except psycopg2.Error, e:
                 _logger.debug("Error issuing pg_stop_backup() command: %s", e)
@@ -624,8 +684,9 @@ class Server(object):
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                cur.execute('SELECT pgespresso_stop_backup(%s)', (backup_label,))
-                return cur.fetchone()[0]
+                cur.execute('SELECT pgespresso_stop_backup(%s), now()',
+                            (backup_label,))
+                return cur.fetchone()
             except psycopg2.Error, e:
                 _logger.debug(
                     "Error issuing pgespresso_stop_backup() command: %s", e)
@@ -640,26 +701,25 @@ class Server(object):
         :param backup_info: backup_info object
         :return:
         """
-        if self.config.backup_options != 'concurrent_backup':
-            stop_string = self.pg_stop_backup()
-            if stop_string:
-                stop_xlog, stop_file_name, stop_file_offset = stop_string
-                backup_info.set_attribute("end_time", datetime.datetime.now())
+        if BackupOptions.CONCURRENT_BACKUP not in self.config.backup_options:
+            stop_row = self.pg_stop_backup()
+            if stop_row:
+                stop_xlog, stop_file_name, stop_file_offset, stop_time = \
+                    stop_row
+                backup_info.set_attribute("end_time", stop_time)
                 backup_info.set_attribute("end_xlog", stop_xlog)
                 backup_info.set_attribute("end_wal", stop_file_name)
                 backup_info.set_attribute("end_offset", stop_file_offset)
             else:
-                self.current_action = "starting the backup: PostgreSQL server " \
-                                      "" \
-                                      "is already in exclusive backup mode"
-                raise Exception('concurrent exclusive backups are not allowed')
-
-        elif self.config.backup_options == 'concurrent_backup':
-
-            end_wal = self.pgespresso_stop_backup(backup_info.backup_label)
-            if end_wal:
+                raise Exception('Cannot terminate exclusive backup. You might '
+                        'have to manually execute pg_stop_backup() on your '
+                        'Postgres server')
+        else:
+            stop_row = self.pgespresso_stop_backup(backup_info.backup_label)
+            if stop_row:
+                end_wal, stop_time = stop_row
                 decoded_segment = xlog.decode_segment_name(end_wal)
-                backup_info.set_attribute("end_time", datetime.datetime.now())
+                backup_info.set_attribute("end_time", stop_time)
                 backup_info.set_attribute("end_xlog",
                                           "%X/%X" % (decoded_segment[1],
                                                      (decoded_segment[
@@ -667,7 +727,9 @@ class Server(object):
                 backup_info.set_attribute("end_wal", end_wal)
                 backup_info.set_attribute("end_offset", 0)
             else:
-                raise Exception('Concurrent stop backup failed')
+                raise Exception('Cannot terminate exclusive backup. You might '
+                        'have to manually execute pg_espresso_abort_backup() '
+                        'on your Postgres server')
 
     def delete_backup(self, backup):
         '''Deletes a backup
@@ -676,7 +738,7 @@ class Server(object):
         '''
         return self.backup_manager.delete_backup(backup)
 
-    def backup(self, immediate_checkpoint):
+    def backup(self):
         """
         Performs a backup for the server
 
@@ -696,7 +758,7 @@ class Server(object):
         try:
             # lock acquisition and backup execution
             with LockFile(filename, raise_if_fail=True):
-                self.backup_manager.backup(immediate_checkpoint)
+                self.backup_manager.backup()
 
         except LockFileBusy:
             output.error("Another backup process is running")
@@ -785,7 +847,7 @@ class Server(object):
     def get_required_xlog_files(self, backup, target_tli=None, target_time=None,
                                 target_xid=None):
         """
-        Get the xlog files required for a backup
+        Get the xlog files required for a recovery
         """
         begin = backup.begin_wal
         end = backup.end_wal
@@ -795,27 +857,30 @@ class Server(object):
             target_tli, _, _ = xlog.decode_segment_name(end)
         with self.xlogdb() as fxlogdb:
             for line in fxlogdb:
-                name, _, stamp, _ = self.xlogdb_parse_line(line)
-                if name < begin: continue
-                tli, _, _ = xlog.decode_segment_name(name)
-                if tli > target_tli: continue
-                yield name
-                if name > end:
-                    end = name
-                    if target_time and target_time < stamp:
+                wal_info = WalFileInfo.from_xlogdb_line(self, line)
+                if wal_info.name < begin:
+                    continue
+                tli, _, _ = xlog.decode_segment_name(wal_info.name)
+                if tli > target_tli:
+                    continue
+                yield wal_info.name
+                if wal_info.name > end:
+                    end = wal_info.name
+                    if target_time and target_time < wal_info.time:
                         break
             # return all the remaining history files
             for line in fxlogdb:
-                name, _, stamp, _ = self.xlogdb_parse_line(line)
-                if xlog.is_history_file(name):
-                    yield name
+                wal_info = WalFileInfo.from_xlogdb_line(self, line)
+                if xlog.is_history_file(wal_info.name):
+                    yield wal_info.name
 
     # TODO: merge with the previous
     def get_wal_until_next_backup(self, backup):
         """
         Get the xlog files between backup and the next
 
-        :param backup: a backup object, the starting point to retrieve wals
+        :param BackupInfo backup: a backup object, the starting point
+            to retrieve WALs
         """
         begin = backup.begin_wal
         next_end = None
@@ -825,15 +890,30 @@ class Server(object):
 
         with self.xlogdb() as fxlogdb:
             for line in fxlogdb:
-                name, size, _, _ = self.xlogdb_parse_line(line)
-                if name < begin: continue
-                tli, _, _ = xlog.decode_segment_name(name)
-                if tli > backup_tli: continue
-                if not xlog.is_wal_file(name): continue
-                if next_end and name > next_end:
+                wal_info = WalFileInfo.from_xlogdb_line(self, line)
+                if wal_info.name < begin:
+                    continue
+                tli, _, _ = xlog.decode_segment_name(wal_info.name)
+                if tli > backup_tli:
+                    continue
+                if not xlog.is_wal_file(wal_info.name):
+                    continue
+                if next_end and wal_info.name > next_end:
                     break
-                # count
-                yield (name, size)
+                yield wal_info
+
+    def get_wal_full_path(self, wal_name):
+        """
+        Build the full path of a WAL for a server given the name
+
+        :param wal_name: WAL file name
+        """
+        # Build the path which contains the file
+        hash_dir = os.path.join(self.config.wals_directory,
+                                xlog.hash_dir(wal_name))
+        # Build the WAL file full path
+        full_path = os.path.join(hash_dir, wal_name)
+        return full_path
 
     def get_wal_info(self, backup_info):
         """
@@ -841,22 +921,67 @@ class Server(object):
 
         :param BackupInfo backup_info: the target backup
         """
+        begin = backup_info.begin_wal
         end = backup_info.end_wal
 
         # counters
         wal_info = dict.fromkeys(
             ('wal_num', 'wal_size',
-             'wal_until_next_num', 'wal_until_next_size'), 0)
+             'wal_until_next_num', 'wal_until_next_size',
+             'wal_until_next_compression_ratio',
+             'wal_compression_ratio'), 0)
+        # First WAL (always equal to begin_wal) and Last WAL names and ts
+        wal_info['wal_first'] = None
+        wal_info['wal_first_timestamp'] = None
         wal_info['wal_last'] = None
+        wal_info['wal_last_timestamp'] = None
+        # WAL rate (default 0.0 per second)
+        wal_info['wals_per_second'] = 0.0
 
-        for name, size in self.get_wal_until_next_backup(backup_info):
-                if name <= end:
+        for item in self.get_wal_until_next_backup(backup_info):
+                if item.name == begin:
+                    wal_info['wal_first'] = item.name
+                    wal_info['wal_first_timestamp'] = item.time
+                if item.name <= end:
                     wal_info['wal_num'] += 1
-                    wal_info['wal_size'] += size
+                    wal_info['wal_size'] += item.size
                 else:
                     wal_info['wal_until_next_num'] += 1
-                    wal_info['wal_until_next_size'] += size
-                wal_info['wal_last'] = name
+                    wal_info['wal_until_next_size'] += item.size
+                wal_info['wal_last'] = item.name
+                wal_info['wal_last_timestamp'] = item.time
+
+        # Estimate WAL ratio
+        if wal_info['wal_last_timestamp']:
+            # Calculate the difference between the timestamps of
+            # the first WAL (begin of backup) and the last WAL
+            # associated to the current backup
+            wal_info['wal_total_seconds'] = (wal_info['wal_last_timestamp'] -
+                                             wal_info['wal_first_timestamp'])
+            if wal_info['wal_total_seconds'] > 0:
+                wal_info['wals_per_second'] = (float(wal_info['wal_num']) /
+                                               wal_info['wal_total_seconds'])
+
+        # evaluation of compression ratio for basebackup WAL files
+        wal_info['wal_theoretical_size'] = \
+            wal_info['wal_num'] * float(xlog.XLOG_SEG_SIZE)
+        try:
+            wal_info['wal_compression_ratio'] = 1 - (
+                wal_info['wal_size'] /
+                wal_info['wal_theoretical_size'])
+        except ZeroDivisionError:
+            wal_info['wal_compression_ratio'] = 0.0
+
+        # evaluation of compression ratio of WAL files
+        wal_info['wal_until_next_theoretical_size'] = \
+            wal_info['wal_until_next_num'] * float(xlog.XLOG_SEG_SIZE)
+        try:
+            wal_info['wal_until_next_compression_ratio'] = 1 - (
+                wal_info['wal_until_next_size'] /
+                wal_info['wal_until_next_theoretical_size'])
+        except ZeroDivisionError:
+            wal_info['wal_until_next_compression_ratio'] = 0.0
+
         return wal_info
 
     def recover(self, backup, dest, tablespaces=[], target_tli=None, target_time=None, target_xid=None, target_name=None, exclusive=False, remote_command=None):
@@ -896,41 +1021,31 @@ class Server(object):
         if not os.path.exists(self.config.wals_directory):
             os.makedirs(self.config.wals_directory)
         xlogdb = os.path.join(self.config.wals_directory, self.XLOG_DB)
-        # If the file doesn't exist and it is required to read it,
-        # we open it in a+ mode, to be sure it will be created
-        if not os.path.exists(xlogdb) and mode.startswith('r'):
-            if '+' not in mode:
-                mode = "a%s+" % mode[1:]
-            else:
-                mode = "a%s" % mode[1:]
+
         xlogdb_lock = xlogdb + ".lock"
         with LockFile(xlogdb_lock, wait=True):
+            # If the file doesn't exist and it is required to read it,
+            # we open it in a+ mode, to be sure it will be created
+            if not os.path.exists(xlogdb) and mode.startswith('r'):
+                if '+' not in mode:
+                    mode = "a%s+" % mode[1:]
+                else:
+                    mode = "a%s" % mode[1:]
+
             with open(xlogdb, mode) as f:
 
                 # execute the block nested in the with statement
-                yield f
+                try:
+                    yield f
 
-                # we are exiting the context
-                # make sure the data is written to disk
-                # http://docs.python.org/2/library/os.html#os.fsync
-                f.flush()
-                os.fsync(f.fileno())
-
-    def xlogdb_parse_line(self, line):
-        '''Parse a line from xlog catalogue
-
-        :param line: a line in the wal database to parse
-        '''
-        try:
-            name, size, stamp, compression = line.split()
-        except ValueError:
-            # Old format compatibility (no compression)
-            compression = None
-            try:
-                name, size, stamp = line.split()
-            except ValueError:
-                raise ValueError("cannot parse line: %r" % (line,))
-        return name, int(size), float(stamp), compression
+                finally:
+                    # we are exiting the context
+                    # if file is writable (mode contains w, a or +)
+                    # make sure the data is written to disk
+                    # http://docs.python.org/2/library/os.html#os.fsync
+                    if any((c in 'wa+') for c in f.mode):
+                        f.flush()
+                        os.fsync(f.fileno())
 
     def report_backups(self):
         if not self.enforce_retention_policies:
