@@ -18,6 +18,7 @@
 from datetime import datetime, timedelta
 import mock
 from barman.infofile import BackupInfo, Tablespace
+from dateutil import tz
 
 
 def mock_backup_info(backup_id='1234567890',
@@ -45,9 +46,9 @@ def mock_backup_info(backup_id='1234567890',
                      timeline=1,
                      version=90302):
     if begin_time is None:
-        begin_time = datetime.now() - timedelta(minutes=10)
+        begin_time = datetime.now(tz.tzlocal()) - timedelta(minutes=10)
     if end_time is None:
-        end_time = datetime.now()
+        end_time = datetime.now(tz.tzlocal())
 
     # Generate a list of tablespace objects
     tablespaces = [Tablespace._make(item) for item in tablespaces]
@@ -72,8 +73,14 @@ def mock_backup_ext_info(backup_info=None,
                           wal_size=123456,
                           wal_until_next_num=18,
                           wal_until_next_size=2345678,
-                          wal_last=000000010000000000000014,
+                          wals_per_second=0.01,
+                          wal_first='000000010000000000000014',
+                          wal_first_timestamp=None,
+                          wal_last='000000010000000000000014',
+                          wal_last_timestamp=None,
                           retention_policy_status=None,
+                          wal_compression_ratio=0.0,
+                          wal_until_next_compression_ratio=0.0,
                           **kwargs):
 
     # make a dictionary with all the arguments
