@@ -25,7 +25,8 @@ import os
 import pwd
 import grp
 import json
-from barman.retention_policies import RetentionPolicy
+import datetime
+
 
 _logger = logging.getLogger(__name__)
 
@@ -138,6 +139,50 @@ def pretty_size(size, unit=1024):
             size /= unit
 
 
+def human_readable_timedelta(timedelta):
+    """
+    Given a time interval, returns a human readable string
+
+    :param timedelta: the timedelta to transform in a human readable form
+    """
+    delta = abs(timedelta)
+    # Calculate time units for the given interval
+    time_map = {
+        'day': int(delta.days % 365),
+        'hour': int(delta.seconds / 3600),
+        'minute': int(delta.seconds / 60) % 60,
+    }
+    # Build the resulting string
+    time_list = []
+
+    # 'Day' part
+    if time_map['day'] > 0:
+        if time_map['day'] == 1:
+            time_list.append('%s day' % time_map['day'])
+        else:
+            time_list.append('%s days' % time_map['day'])
+
+    # 'Hour' part
+    if time_map['hour'] > 0:
+        if time_map['hour'] == 1:
+            time_list.append('%s hour' % time_map['hour'])
+        else:
+            time_list.append('%s hours' % time_map['hour'])
+
+    # 'Minute' part
+    if time_map['minute'] > 0:
+        if time_map['minute'] == 1:
+            time_list.append('%s minute' % time_map['minute'])
+        else:
+            time_list.append('%s minutes' % time_map['minute'])
+
+    human = ', '.join(time_list)
+    # If timedelta is negative append 'ago' suffix
+    if delta != timedelta:
+        human += " ago"
+    return human
+
+
 class BarmanEncoder(json.JSONEncoder):
     """
     Custom JSON encoder used for BackupInfo encoding
@@ -145,16 +190,23 @@ class BarmanEncoder(json.JSONEncoder):
     This encoder supports the following types:
 
     * dates and timestamps if they have a ctime() method.
-    * RetentionPolicy objects
+    * objects that implement the 'to_json' method.
     * binary strings (python 3)
 
     """
     def default(self, obj):
+        # If the object implements to_json() method use it
+        if hasattr(obj, 'to_json'):
+            return obj.to_json()
+        # Serialise date and datetime objects using ctime() method
         if hasattr(obj, 'ctime') and callable(obj.ctime):
             return obj.ctime()
-        if isinstance(obj, RetentionPolicy):
-            return "%s %s" % (obj.mode, obj.value)
+        # Serialise  timedelta objects using human_readable_timedelta()
+        if isinstance(obj, datetime.timedelta):
+            return human_readable_timedelta(obj)
+        # Binary strings must be decoded before using them in
+        # an unicode string
         if hasattr(obj, 'decode') and callable(obj.decode):
             return obj.decode('utf-8', 'replace')
         # Let the base class default method raise the TypeError
-        return super(BarmanEncoder, self).default(obj)
\ No newline at end of file
+        return super(BarmanEncoder, self).default(obj)
